/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package kelp.webapp.presentation;

import org.w3c.dom.Node;
import org.w3c.dom.html.HTMLElement;

import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import java.io.BufferedReader;
import java.io.ByteArrayInputStream;
import java.io.IOException;
import java.io.File;
import java.io.FileInputStream;
import java.io.InputStreamReader;

/**
 * <p>
 * This presentation object dynamically creates an HTML page
 * showing the greeting from the application configuration file.
 * </p>
 */
public class NewNodeServlet extends HttpServlet {
  private NewNodeHTML page = null;

  public void doGet(HttpServletRequest request, HttpServletResponse response)
        throws ServletException, IOException
    {
    page = createPage();
    response.getWriter().write(page.toDocument());
    response.getWriter().flush();
    response.flushBuffer();
  }

  public NewNodeHTML createPage()  {
    page = new NewNodeHTML();
    fixupSpan();
    return page;
  }

  private void fixupSpan() {
    HTMLElement element  = page.getElementSpanInsertHere();
    String      fromFile = readFile();
    Node        newNode  = page.createCDATASection(fromFile);
    element.getParentNode().replaceChild(newNode, element);
  }

  private String readFile() {
    File                 nodeFile   = null;
    FileInputStream      fileStream = null;
    InputStreamReader    inReader   = null;
    BufferedReader       buffer     = null;
    ByteArrayInputStream byteIn     = null;

    StringBuffer outBuf = new StringBuffer();
    String inLine  = null;

    StringBuffer fileNameBuf = new StringBuffer();
    fileNameBuf.append(getInitParameter("testDataPath"));
      if (fileNameBuf.length() == 0) {
        outBuf.append("<p>init-parm not found: testDataPath</p>");
      } else {
        fileNameBuf.append(File.separator);
        fileNameBuf.append("newnode.txt");
        try {
          nodeFile = new File(fileNameBuf.toString());
          fileStream = new FileInputStream(nodeFile);
          inReader   = new InputStreamReader(fileStream);
          buffer     = new BufferedReader(inReader);
          inLine = new String();
          while (inLine != null) {
            inLine = buffer.readLine();
            if (inLine != null) {
              outBuf.append(inLine);
            }
          }
          buffer.close();
          inReader.close();
          fileStream.close();
        } catch (IOException e) {
          outBuf.append(e.getMessage());
        }
      }
    return outBuf.toString();
  }


}


