/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id$
 */


package org.enhydra.kelp.common.bridge;

// Kelp imports
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterXMLCNode;
import org.enhydra.kelp.common.node.OtterProject;

// XMLC imports
import org.enhydra.xml.xmlc.dom.XMLCDocument;
import org.enhydra.xml.xmlc.XMLCException;
import org.enhydra.xml.xmlc.compiler.EditDOM;
import org.enhydra.xml.xmlc.metadata.MetaData;

// Standard imports
import java.io.PrintWriter;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;

public class XMLCConnectionFactory {

  public XMLCConnectionFactory() {
  }

  public static MetaDataHandler createMetaDataHandler() {

    MetaDataHandler handler = null;
    if (KelpVersion.getEnhydraVersion().startsWith("3")) {
      handler = new MetaDataHandlerV3();
    } else {
      handler = new MetaDataHandlerV4();
    }
    return handler;
  }

  public static EditDOM createEditDOM(MetaDataHandler h) {
    final String CLASSNAME_EDITDOM =
       "org.enhydra.xml.xmlc.compiler.EditDOM";

    EditDOM editDOM = null;

    if (KelpVersion.getEnhydraVersion().startsWith("3")) {
       Class editClass = null;
       Class[] params = new Class[0];
       Constructor construct = null;
       Method[] meths = new Method[0];
       Object[] values = new Object[0];

       try {
         editClass = Class.forName(CLASSNAME_EDITDOM);
         construct = editClass.getConstructor(params);
         editDOM = (EditDOM) construct.newInstance(params);
         meths = editClass.getMethods();
         for (int i = 0 ; i < meths.length ; i++ ) {
           if (meths[i].getName().equalsIgnoreCase("addURLEdits")) {
             values = new Object[1];
             values[0] = h.getURLMappings();
             meths[i].invoke(editDOM, values);
             break;
           }
         }
         for (int i = 0 ; i < meths.length ; i++ ) {
           if (meths[i].getName().equalsIgnoreCase("addDeleteClasses")) {
             values = new Object[1];
             values[0] = h.getDeleteElements();
             meths[i].invoke(editDOM, values);
             break;
           }
         }

       } catch (Exception e) {
         editClass = null;
         e.printStackTrace();
       }
    } else {
      editDOM = new EditDOM((MetaData) h.getMetaData());
    }
    return editDOM;
  }

  public static Generator createGenerator(MetaDataHandler h,
       XMLCDocument doc, PrintWriter writer)
         throws XMLCException  {
    Generator gen = null;
    if (KelpVersion.getEnhydraVersion().startsWith("3")) {
      gen = new GeneratorV3(h, doc, writer);
    } else {
      gen = new GeneratorV4(h, doc, writer);
    }
    return gen;
  }


  public static Parser createParser(PrintWriter traceWriter, PrintWriter infoWriter) {
    Parser parser = null;
    if (KelpVersion.getEnhydraVersion().startsWith("3")) {
      parser = new ParserV3(traceWriter, infoWriter);
    } else {
      parser = new ParserV4(traceWriter, infoWriter);
    }
    return parser;

  }

}
