/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id$
 */
package org.enhydra.kelp.common.node;

// Kelp imports
import org.enhydra.kelp.common.bridge.XMLCConnectionFactory;
import org.enhydra.kelp.common.bridge.MetaDataHandler;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterProject;

// Enhydra imports
import org.enhydra.xml.xmlc.compiler.Parse;
import org.enhydra.xml.xmlc.XMLCException;

// Standard imports
import java.io.File;
import java.io.IOException;
import java.io.PrintWriter;
import java.util.Vector;
import java.util.StringTokenizer;
import java.lang.reflect.Constructor;
import java.lang.reflect.Method;
import java.lang.reflect.InvocationTargetException;

/**
 * Class declaration
 *
 *
 * @author Paul Mahar
 * @version $Revision$
 */
public class OtterXMLCNode implements OtterResourceNode {
    public final static int   CLASS_NAME_DEFAULT = 1;
    public final static int   CLASS_NAME_CUSTOM = 2;
    public final static int   CLASS_NAME_MAPPED = 3;
    private final String      CUSTOM_CLASS_NAME =
        "enhydra.xmlc.customClassName";
    private final String      CLASS_NAME_TYPE = "enhydra.xmlc.classNameType";
    private MetaDataHandler   metaDataHandler;
    private OtterResourceNode resourceNode;

    /**
     * Create a node to use when calling XMLC.
     *
     */
    public OtterXMLCNode(OtterResourceNode r) {
        resourceNode = r;
        initMetaDataHandler();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCOptionFilename() {
        return resourceNode.getXMLCOptionFilename();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setXMLCOptionFilename(String n) {
        resourceNode.setXMLCOptionFilename(n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getXMLCParameters() {
        return resourceNode.getXMLCParameters();
    }

    /**
     * Method declaration
     *
     *
     * @param p
     *
     * @see
     */
    public void setXMLCParameters(String p) {
        resourceNode.setXMLCParameters(p);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public boolean isSelected() {
        return resourceNode.isSelected();
    }

    /**
     * Method declaration
     *
     *
     * @param b
     *
     * @see
     */
    public void setSelected(boolean b) {
        resourceNode.setSelected(b);
        if (b) {
            setStatic(false);
        }
    }


    public boolean isStatic() {
        return resourceNode.isStatic();
    }

    public void setStatic(boolean b) {
        resourceNode.setStatic(b);
        if (b) {
            setSelected(false);
        }
    }


    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public char getFileSeparatorChar() {
        return resourceNode.getFileSeparatorChar();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getFilePath() {
        return resourceNode.getFilePath();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getCustomClassName() {
        String custom = getProperty(CUSTOM_CLASS_NAME);

        if (custom == null) {
            custom = new String();
        }
        return custom;
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @see
     */
    public void setCustomClassName(String n) {
        setProperty(CUSTOM_CLASS_NAME, n);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public Object getNativeNode() {
        return resourceNode.getNativeNode();
    }

    /**
     * Method declaration
     *
     *
     * @param o
     *
     * @see
     */
    public void setNativeNode(Object o) {
        resourceNode.setNativeNode(o);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public OtterProject getProject() {
        return resourceNode.getProject();
    }

    /**
     * Method declaration
     *
     *
     * @param n
     *
     * @return
     *
     * @see
     */
    public String getProperty(String n) {
        return resourceNode.getProperty(n);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     *
     * @see
     */
    public void setProperty(String n, String v) {
        resourceNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @param n
     * @param v
     *
     * @see
     */
    public void setProperty(String n, int v) {
        resourceNode.setProperty(n, v);
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public int getClassNameType() {

        // When getting the project scope overrides
        // local type.
        int    type;
        int    projectScope;
        String stringType = getProperty(CLASS_NAME_TYPE);

        projectScope = getProject().getMapScope();
        type = PropUtil.stringToInt(stringType, CLASS_NAME_MAPPED);
        switch (type) {
        case CLASS_NAME_DEFAULT:
        case CLASS_NAME_CUSTOM:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                type = CLASS_NAME_MAPPED;
                setProperty(CLASS_NAME_TYPE, type);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                type = CLASS_NAME_DEFAULT;
                setProperty(CLASS_NAME_TYPE, type);
            }
            break;
        }
        return type;
    }

    /**
     * Method declaration
     *
     *
     * @param newType
     *
     * @see
     */
    public void setClassNameType(int newType) {

        // If conflict exists with project scope,
        // change project scope.
        int oldType = getClassNameType();
        int projectScope = getProject().getMapScope();

        setProperty(CLASS_NAME_TYPE, newType);
        syncProjectToType(newType);
        if ((newType != oldType) && (newType != CLASS_NAME_CUSTOM)) {
            initClassName();
        }
    }

    /**
     * Method declaration
     *
     *
     * @param forPrint
     *
     * @return
     *
     * @see
     */
    private String[] getAllParameters(boolean forPrint) {
        StringBuffer    buf = new StringBuffer();
        String[]        parameters = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        // Project
        if (resourceNode.getProject().getXMLCParameters() != null) {
            buf.append(resourceNode.getProject().getXMLCParameters().trim());
        }
        if (forPrint) {
            buf.append("#");
        } else {
            buf.append(" ");
        }
        // Package
        OtterNode packNode = null;
        String packParam = new String();
        packNode = resourceNode.getParent();
       if (packNode == null || packNode instanceof OtterProject) {
          // skip it.
       } else {
          packParam = packNode.getXMLCParameters();
       }
        if ((packParam != null) && (packParam.trim().length() > 0)) {
          buf.append(packParam);
          if (forPrint) {
              buf.append("#");
          } else {
              buf.append(" ");
          }
        }

        // Node
        if (getXMLCParameters() != null) {
            buf.append(getXMLCParameters().trim());
        }
        if (forPrint) {
            tokenizer = new StringTokenizer(buf.toString().trim(), "#");
        } else {
            tokenizer = new StringTokenizer(buf.toString().trim());
        }
        count = tokenizer.countTokens();
        parameters = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            parameters[index] = tokenizer.nextToken();
            index++;
        }
        return parameters;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String[] getAllOptionFilenames() {
        StringBuffer    buf = new StringBuffer();
        File            file = null;
        String          filename  = new String();
        String[]        filenames = new String[0];
        StringTokenizer tokenizer = null;
        int             count = 0;
        int             index = 0;

        // Project
        filename = resourceNode.getProject().getXMLCOptionFilename();
        if (filename != null && filename.trim().length() > 0) {
            // must insert project first so it can be overriden
            file =  new File(filename);
            if (file.exists() && file.isFile()
                    && file.getName().toLowerCase().endsWith(".xmlc")) {
                buf.append(file.getAbsolutePath());
            }
        }
        // Package
        OtterNode parent = resourceNode.getParent();
        if (parent == null || parent instanceof OtterProject) {
          // skip it
        } else {

           filename = parent.getXMLCOptionFilename();
           if (filename != null && filename.trim().length() > 0) {
              file = new File(filename);
              if (file.exists() && file.isFile()
                      && file.getName().toLowerCase().endsWith(".xmlc")) {
                  buf.append("#");
                  buf.append(file.getAbsolutePath());
              }
          }
        }

        buf.append("#");
        // Node
        filename = getXMLCOptionFilename();
        if (filename != null && filename.trim().length() > 0) {
            file = new File(filename);
            if (file.exists() && file.isFile()
                    && file.getName().toLowerCase().endsWith(".xmlc")) {
                buf.append(file.getAbsolutePath());
            }
        }
        tokenizer = new StringTokenizer(buf.toString().trim(), "#");
        count = tokenizer.countTokens();
        filenames = new String[tokenizer.countTokens()];
        while (tokenizer.hasMoreTokens()) {
            filenames[index] = tokenizer.nextToken();
            index++;
        }
        return filenames;
    }

    /**
     * Method declaration
     *
     *
     * @param type
     *
     * @see
     */
    private void syncProjectToType(int type) {
        int projectScope = getProject().getMapScope();

        switch (type) {
        case CLASS_NAME_CUSTOM:
        case CLASS_NAME_DEFAULT:
            if (projectScope == OtterProject.MAP_SCOPE_ALL) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        case CLASS_NAME_MAPPED:
            if (projectScope == OtterProject.MAP_SCOPE_NONE) {
                getProject().setMapScope(OtterProject.MAP_SCOPE_SELECTED);
            }
            break;
        }
    }

    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
        if (e == null) {
            resourceNode.setException(null);
        } else {
            resourceNode.setException(new LocalException(e));
        }
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
        return resourceNode.getException();
    }

    public OtterNode getParent() {
      return resourceNode.getParent();
    }


    /**
     * Get the XMLC options selected this HTML/WML file.
     */
    public MetaDataHandler getMetaDataHandler() {
        return metaDataHandler;
    }

    /**
     * Add the generated Java source file to the IDE project. Do not call
     * this when using Win32 JBuilder as it can't handle the width of source
     * lines generated by XMLC.
     */
    public void replaceGeneratedSource(boolean forRecomp) {
        if ((metaDataHandler.getJavaClassSource().exists())
                && (getProject().isOpenBuild())) {
            int count = 1;
            if (forRecomp) {
              count = 2;
            }
            String[] source = new String[count];
            OtterNodeFactory factory = null;
            source[0] = metaDataHandler.getJavaClassSource().getAbsolutePath();
            factory = getProject().getNodeFactory();

            if (forRecomp) {
              source[1] = metaDataHandler.getJavaInterfaceSource().getAbsolutePath();
            }
            factory.replaceGeneratedSource(getProject(), resourceNode,
                                           source);

        }
    }


    /**
     * Method declaration
     *
     *
     * @see
     */
    public void initProjectOptions() {
        metaDataHandler.setKeepGeneratedSource(true);         // always keep Java source
        metaDataHandler.setCompileSource(false);   // never compile Java source

        // loging options
        metaDataHandler.setPrintParseInfo(getProject().isParseInfoSelected());
        metaDataHandler.setPrintParseInfo(getProject().isDumpSelected());
        metaDataHandler.setVerbose(getProject().isVerboseSelected());
        metaDataHandler.setPrintAccessorInfo(getProject().isPrintAccessMethodsSelected());

        // Call XMLC with the do not call javac option.
    }

    public String getGenerateToRoot() {
        String       projectGenTo = getProject().getGenerateToDirectory();
        StringBuffer buf = new StringBuffer();
        if (projectGenTo == null) {
            buf.append(resourceNode.getProject().getSourcePathOf(resourceNode));
        } else {
            buf.append(projectGenTo);
        }
        return buf.toString();
    }


    /**
     * Initialize the XMLC options.
     *
     * @throws XMLCException
     * Thrown when options are invalid.
     *
     */
    private void initMetaDataHandler() {
        metaDataHandler = XMLCConnectionFactory.createMetaDataHandler();
        initClassName();
        initProjectOptions();
        metaDataHandler.setInputDocument(getFilePath());
        initJavaSourceFile();
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    private void parseOptions() throws XMLCException, IOException {
        metaDataHandler.parse(getAllOptionFilenames(),
                              getAllParameters(false),
                              (PrintWriter) null,
                              this);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void initJavaSourceFile() {
        StringBuffer filename = new StringBuffer();
        int          index = -1;
        filename.append(getGenToDirectory().toString());
        filename.append("" + getProject().getFileSeparatorChar());
        String className = metaDataHandler.getClassName();
        if (metaDataHandler.getPackageName() != null) {
            index = className.indexOf(metaDataHandler.getPackageName());
        }

        if (index > -1) {
            className = className.substring(metaDataHandler.getPackageName().length()
                                            + 1);
        }
        index = className.lastIndexOf(".");
        if (index > -1) {
            className = className.substring(index + 1);
        }
        filename.append(className);
        filename.append(".java");
        File f = new File(filename.toString());
        metaDataHandler.setJavaClassSource(f, this);
    }

    /**
     * Get the generate to directory and create the directory if needed.
     * This is where the Java source files are created.
     */
    private File getGenToDirectory() {
        String genTo = getGenToPath();
        File   genDir = new File(genTo);

        if (genDir.exists()) {
            if (!genDir.isDirectory()) {
                System.err.println("Unable to write to directory: " + genTo);
            }
        } else {
            genDir.mkdirs();
        }
        return genDir;
    }


    /**
     * Get the generate to directory as a string. This is where XMLC will create
     * Java source files.
     */
    private String getGenToPath() {
        String       packagePath = getPackagePath();
        StringBuffer buf = new StringBuffer();
        buf.append(getGenerateToRoot());
        if (packagePath.length() > 0) {
            buf.append(File.separator);
            buf.append(packagePath);
        }
        return buf.toString();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    private String getPackagePath() {
        String packPath = new String();
        if (metaDataHandler.getPackageName() != null) {
            packPath = metaDataHandler.getPackageName();
        } else if (metaDataHandler.getClassName() != null) {
            int index = metaDataHandler.getClassName().lastIndexOf('.');
            if (index > -1) {
                packPath = metaDataHandler.getClassName().substring(0, index);
            }
        }
        packPath = packPath.replace('.', File.separatorChar);
        return packPath;
    }

    /**
     * Create a class name for the given HTML/WML file and map the
     * package name as needed. Mapping allows users to  specify
     * a package name to use for all the HTML/WML files in a directory.
     */
    private void initClassName() {
        Mapper mapper = new Mapper(getProject());
        String className = mapper.getClassName(this);
        metaDataHandler.setClassName(className);
    }

    private String baseOut() {
        StringBuffer buf = new StringBuffer();
        String path = new String();
        buf.append(getProject().getClassOutputPath());
        buf.append(File.separator);
        path = metaDataHandler.getClassName();
        path = path.replace('.', File.separatorChar);
        buf.append(path);
        return buf.toString();
    }

    /**
     * Get the class file that we expect after XMLC has generated
     * a java file and called javac to compile it.
     */
    public File getClassFile() {
        File file = new File(baseOut() + ".class");
        return file;
    }
    public File getOptionFileForRecomp() {
       File file = new File(baseOut() + ".xmlc");
       return file;
    }

    /**
     * True if we can find the Java source file that we expect XMLC to create.
     * This is used to in reporting compilation results.
     */
    public boolean isNewJavaFound() {
        boolean newFound = false;

        if (metaDataHandler.getKeepGeneratedSource()) {
            newFound = metaDataHandler.getJavaClassSource().exists();
        }
        return newFound;
    }

    /**
     * Method declaration
     *
     *
     * @exception IOException
     * @exception XMLCException
     *
     * @see
     */
    public void preCompile() throws XMLCException, IOException {
        parseOptions();

        // clear out any existing impl or standard classes.
        File implSource = null;
        StringBuffer implName = new StringBuffer();
        int index = -1;
        implName.append(metaDataHandler.getJavaClassSource());
        index = implName.toString().lastIndexOf(".");
        implName.insert(index, "Impl");
        implSource = new File(implName.toString());
        if (implSource.exists()) {
          implSource.delete();
        }
        if (metaDataHandler.getJavaClassSource().exists()) {
          metaDataHandler.getJavaClassSource().delete();
        }

        // reset names if needed
        if (metaDataHandler.getRecompilation()) {
          metaDataHandler.setJavaInterfaceSource(metaDataHandler.getJavaClassSource(), this);
          metaDataHandler.setJavaClassSource(implSource, this);
        }
        resourceNode.preCompile();
    }


    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision$
     */
    class LocalException extends Exception {
        private Exception baseException = null;

        /**
         * Constructor declaration
         *
         *
         * @param e
         *
         * @see
         */
        public LocalException(Exception e) {
            baseException = e;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        public String getMessage() {
            StringBuffer buf = new StringBuffer();
            String[]     parameters = getAllParameters(true);
            String[]     filenames = getAllOptionFilenames();

            buf.append(baseException.getMessage());
            buf.append("\n");
            if (parameters.length > 0) {
                buf.append("  Command line parameters: \n");
                for (int i = 0; i < parameters.length; i++) {
                    buf.append("     ");
                    buf.append(parameters[i]);
                    buf.append("\n");
                }
            }
            if (filenames.length > 0) {
                buf.append("  Option file");
                if (filenames.length > 1) {
                    buf.append("s");
                }
                buf.append(":\n");
                for (int i = 0; i < filenames.length; i++) {
                    buf.append("     ");
                    buf.append(filenames[i]);
                    buf.append("");
                }
            }
            return buf.toString().trim();
        }

    }
}
