/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */
package org.enhydra.kelp.common.wizard;
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.capture.Writer;
import org.enhydra.kelp.common.capture.WriteEvent;
import org.enhydra.kelp.common.capture.WriteListener;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterSelectionNode;
import org.enhydra.kelp.common.swing.AboutBox;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.xml.xmlc.XMLCVersion;
import javax.swing.JButton;
import javax.swing.JDialog;
import javax.swing.JPanel;
import javax.swing.JTabbedPane;
import javax.swing.SwingUtilities;
import java.awt.BorderLayout;
import java.awt.Dialog;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Frame;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.util.Vector;

/**
 * This class defines the main dialog for the XMLC Wizard.
 *
 * @author
 * Paul Mahar
 * @version %I%, %G%
 */
abstract public class AbstractBuildDialog extends JDialog
    implements SelectionPanelListener {

    public SelectionPanel        selectionPanel;

    private boolean               standalone = false;
    private OtterSelectionNode[]  nodes = null;
    private OtterProject          project = null;
    private LocalButtonListener   buttonListener = null;
    private JPanel                panelMain;
    private JPanel                panelTab;
    private JPanel                panelButton;
    private JPanel                panelBottom;
    private JPanel                panelFiller2;
    private JPanel                panelOptionFiller0;
    private JPanel                panelOptionFiller1;
    private JPanel                panelOptionFiller2;
    private GridBagLayout         layoutMain;
    private GridBagLayout         layoutOption;
    private GridBagLayout         layoutButton;
    private GridBagLayout         layoutBottom;
    private JPanel                propertyPanelHolder;
    private AbstractPropertyPanel propertyPanel = null;
    private OutputPanel           outputPanel;
    private JTabbedPane           tabCompiler;
    private JButton               buttonClose;
    private JButton               buttonAbout;
    private Vector                writeListeners = new Vector();
    public JButton                buttonMake;

    public AbstractBuildDialog(Frame owner) {
        super(owner);
    }


    /**
     * Create the dialog as a child of a frame.
     *
     * @param owner
     * The parent Frame used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public AbstractBuildDialog(Frame owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    /**
     * Create the dialog as a child of another dialog.
     *
     * @param owner
     * The parent Dialog used when the dialog is modal.
     * @param title
     * The title for the dialog.
     * @param modal
     * Normally this is set to true for wizards.
     */
    public AbstractBuildDialog(Dialog owner, String title, boolean modal) {
        super(owner, title, modal);
    }

    /**
     * Method declaration
     *
     *
     * @param e
     *
     * @see
     */
    public void onSelection(SelectionPanelEvent e) {
        boolean enable = e.getSelected().size() > 0;

        buttonMake.setEnabled(enable);
    }

    /**
     * Overloaded method that will center the dialog prior to making it
     * visible.
     */
    public void show() {

        // setResizable(false);  May cause crash in Win98
        Point cPoint;

        invalidate();
        doLayout();
        pack();
        cPoint = SwingUtil.getCenteringPoint(getSize());
        setLocation(cPoint);
        super.show();
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public synchronized Vector getWriteListeners() {
        return writeListeners;
    }

    /**
     * Method declaration
     *
     *
     * @param l
     *
     * @see
     */
    public synchronized void addWriteListener(WriteListener l) {
        writeListeners.addElement(l);
    }

    /**
     * Method declaration
     *
     *
     * @param l
     *
     * @see
     */
    public synchronized void removeWriteListener(WriteListener l) {
        if (writeListeners.contains(l)) {
            writeListeners.removeElement(l);
        }
    }

    /**
     * Remove all object references.
     */
    public void clearAll() {
        layoutMain.removeLayoutComponent(panelTab);
        panelTab.removeAll();
        panelMain.removeAll();
        layoutOption = null;
        layoutMain = null;
        panelTab = null;
        panelMain = null;
        setTitle(null);
    }

    /**
     * Get source files that can be selected for compilation.
     */
    public OtterSelectionNode[] getNodes() {
        return nodes;
    }

    /**
     * Set the source files that can be selected for compilation
     */
    public void setNodes(OtterSelectionNode[] n) {
        nodes = n;
        selectionPanel.setNodes(nodes);
    }

    /**
     * Get the IDE project containing the HTML files.
     */
    public OtterProject getProject() {
        return project;
    }

    /**
     * Set the IDE project containing the HTML files.
     */
    public void setProject(OtterProject p) {
        project = p;
        propertyPanel.setProject(project);
        propertyPanel.readProperties();
        outputPanel.setProject(project);
    }

    /**
     * Close the dialog and exit the VM if this wizard
     * was run as a standalone application.
     */
    protected void closeDialog() {
        setVisible(false);
        removeAll();
        dispose();
        if (standalone) {
            System.exit(0);
        }
    }

    /**
     * Select the output to show compilation progress. This
     * is called when XMLC is invoked.
     */
    private void selectOutputTab() {
        Runnable runSwing = new Runnable() {

            /**
             * Method declaration
             *
             *
             * @see
             */
            public void run() {
                tabCompiler.setSelectedComponent(outputPanel);
                SwingUtilities.windowForComponent(tabCompiler).repaint();
            }

        };

        try {
            if (SwingUtilities.isEventDispatchThread()) {
                runSwing.run();
            } else {
                SwingUtilities.invokeAndWait(runSwing);
            }
        } catch (Exception e) {
            e.printStackTrace();
        }
        try {
            Thread.sleep(500);
        } catch (InterruptedException e) {

            //
        }
    }

    /**
     * Construct the object. This is called from both constructors.
     */
    public void init(AbstractPropertyPanel p) {
        propertyPanel = p;
        try {
            jbInit();
            pmInit();
            pack();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    /**
     * Add listeners and other items that are not generated by
     * the JBuilder designer.
     */
    private void pmInit() {
        propertyPanelHolder.add(propertyPanel, BorderLayout.CENTER);
        tabCompiler.add("Selections", selectionPanel);
        tabCompiler.add("Options", panelOptionFiller0);
        tabCompiler.add("Output", outputPanel);
        addWriteListener(outputPanel);
        buttonListener = new LocalButtonListener();
        buttonMake.addActionListener(buttonListener);
        buttonClose.addActionListener(buttonListener);
        buttonAbout.addActionListener(buttonListener);
        selectionPanel.addSelectionPanelListener(this);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    private void showAbout() {
        StringBuffer buf = new StringBuffer();

        buf.append("Enhydra ");
        buf.append(KelpVersion.getEnhydraVersion());
        buf.append("\n");
        buf.append("XMLC ");
        buf.append(XMLCVersion.VERSION);
        AboutBox about = new AboutBox(this);

        about.setTitleText("Kelp " + KelpVersion.VERSION);
        about.setTextAreaText(buf.toString());
        about.setSmallText("Copyright (c) 2000 Lutris Technologies");
        about.show();
    }

    /**
     * UI code generated by JBuilder.
     * <P><I>Do not modify the signature of this method.</I></P>
     */
    private void jbInit() throws Exception {
        panelMain = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                               JPanel.class.getName());
        panelTab = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        panelBottom = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelButton = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        panelFiller2 = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        panelOptionFiller0 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller1 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        panelOptionFiller2 =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutOption =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutButton =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        layoutBottom =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        selectionPanel =
            (SelectionPanel) Beans.instantiate(getClass().getClassLoader(),
                                               SelectionPanel.class.getName());
        propertyPanelHolder =
            (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                       JPanel.class.getName());
        outputPanel =
            (OutputPanel) Beans.instantiate(getClass().getClassLoader(),
                                            OutputPanel.class.getName());
        tabCompiler =
            (JTabbedPane) Beans.instantiate(getClass().getClassLoader(),
                                            JTabbedPane.class.getName());
        buttonMake = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                 JButton.class.getName());
        buttonClose = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonAbout = (JButton) Beans.instantiate(getClass().getClassLoader(),
                                                  JButton.class.getName());
        buttonMake.setText("Make");
        buttonMake.setEnabled(false);
        buttonClose.setText("Close");
        panelTab.setLayout(layoutOption);
        buttonAbout.setText("About...");
        panelTab.add(tabCompiler,
                     new GridBagConstraints(0, 0, 1, 1, 1.0, 1.0,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.BOTH,
                                            new Insets(5, 5, 5, 5), 5, 5));
        panelButton.setLayout(layoutButton);
        panelButton.add(buttonMake,
                        new GridBagConstraints(1, 0, 1, 4, 0.5, 0.5,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonClose,
                        new GridBagConstraints(2, 0, 1, 4, 0.5, 0.5,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(5, 5, 5, 5), 0, 0));
        panelButton.add(buttonAbout,
                        new GridBagConstraints(3, 3, 1, 1, 0.0, 0.0,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.NONE,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.setLayout(new GridBagLayout());
        panelOptionFiller0.add(panelOptionFiller1,
                               new GridBagConstraints(0, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(propertyPanelHolder,
                               new GridBagConstraints(1, 0, 1, 4, 0.7, 0.7,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.NONE,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelOptionFiller0.add(panelOptionFiller2,
                               new GridBagConstraints(2, 0, 1, 1, 0.15, 0.15,
                               GridBagConstraints.CENTER,
                               GridBagConstraints.BOTH,
                               new Insets(0, 0, 0, 0), 0, 0));
        panelBottom.setLayout(layoutBottom);
        panelBottom.add(panelFiller2,
                        new GridBagConstraints(0, 0, 1, 1, 0.4, 0.4,
                                               GridBagConstraints.CENTER,
                                               GridBagConstraints.BOTH,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelBottom.add(panelButton,
                        new GridBagConstraints(1, 0, 1, 4, 0.6, 0.6,
                                               GridBagConstraints.EAST,
                                               GridBagConstraints.NONE,
                                               new Insets(0, 0, 0, 0), 0, 0));
        panelMain.setLayout(layoutMain);
        panelMain.add(panelTab,
                      new GridBagConstraints(0, 0, 1, 1, 0.75, 0.75,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));
        panelMain.add(panelBottom,
                      new GridBagConstraints(0, 1, 1, 1, 0.25, 0.25,
                                             GridBagConstraints.CENTER,
                                             GridBagConstraints.BOTH,
                                             new Insets(0, 0, 0, 0), 0, 0));
        this.getContentPane().add(panelMain);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    abstract public void make();

    /**
     * This inner class defines the actions for buttons in the wizard.
     *
     * @author
     * Paul Mahar
     * @version %I%, %G%
     */
    class LocalButtonListener implements ActionListener {

        /**
         * ActionListener implementation.
         *
         * @param event
         * Event that triggered this listener.
         *
         */
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonMake) {
                selectOutputTab();
                propertyPanel.writeProperties();
                make();
            } else if (source == buttonClose) {
                propertyPanel.writeProperties();
                closeDialog();
            } else if (source == buttonAbout) {
                showAbout();
            }
        }

    }
}

