/**
 * Title:        Kelp
 * Description:  Enhydra tools for Java IDEs
 * Copyright:    Copyright (c) 1999-2000
 * Company:      Lutris
 * @author Lutris Technologies
 * @version
 */
package org.enhydra.kelp.common.wizard.conf;
//
import org.enhydra.kelp.common.wizard.AbstractPropertyPanel;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.swing.SwingUtil;
import java.awt.*;
import java.awt.event.*;
import java.io.File;
import javax.swing.*;
import javax.swing.border.*;
import java.beans.*;
//
public class ResourcePanel extends AbstractPropertyPanel {
    private GridBagLayout       layoutMain;
    private TitledBorder        borderResources;
    private JRadioButton        radioResourceMap;
    private JRadioButton        radioResourcePath;
    private JLabel              labelResource;
    private JTextField          textResource;
    private JButton             buttonResource;
    private ButtonGroup         radioGroup;
    private LocalButtonListener buttonListener = null;
    private LocalRadioListener  radioListener = null;
    private JPanel              panelAPI;
    private GridBagLayout       layoutAPI;
    private JPanel              panelContent;
    private JLabel              labelOutput;
    private JTextField          textOutput;
    private JButton             buttonOutput;
    private GridBagLayout       layoutContent;
    private JPanel              panelFiller;
    private BorderLayout        layoutFiller;

    public ResourcePanel() {
        try {
            jbInit();
            pmInit();
        } catch (Exception ex) {
            ex.printStackTrace();
        }
    }

    public void readProperties() {
        if (project.isResourceMap()) {
            radioResourceMap.setSelected(true);
            radioResourcePath.setSelected(false);
            textResource.setText("");
            textResource.setEnabled(false);
            buttonResource.setEnabled(false);
            textOutput.setText("");
            textOutput.setEnabled(false);
            buttonOutput.setEnabled(false);

        } else {
            radioResourceMap.setSelected(false);
            radioResourcePath.setSelected(true);
            textResource.setText(project.getResourcePath());
            textResource.setEnabled(true);
            buttonResource.setEnabled(true);
            textOutput.setText("");
            textOutput.setEnabled(true);
            buttonOutput.setEnabled(true);

        }
    }

    public void writeProperties() {
        if (radioResourceMap.isSelected()) {
            project.setResourceMap(true);
            project.setResourcePath("");
        } else {
            project.setResourceMap(false);
            project.setResourcePath(textResource.getText());
        }
    }

    private void browseForPath(JTextField field, String caption) {
        File choice = null;

        choice =
            SwingUtil.getDirectoryChoice(this, field.getText(),
                                         caption);
        if (choice != null && choice.isDirectory()) {
            field.setText(choice.getAbsolutePath());
        }
    }

    private void pmInit() {
        buttonListener = new LocalButtonListener();
        buttonResource.addActionListener(buttonListener);
        buttonOutput.addActionListener(buttonListener);
        radioGroup = new ButtonGroup();
        radioGroup.add(radioResourceMap);
        radioGroup.add(radioResourcePath);
        radioListener = new LocalRadioListener();
        radioResourceMap.addActionListener(radioListener);
        radioResourcePath.addActionListener(radioListener);
    }

    private void jbInit() throws Exception {
        layoutMain =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        radioResourceMap =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        radioResourcePath =
            (JRadioButton) Beans.instantiate(getClass().getClassLoader(),
                                             JRadioButton.class.getName());
        labelResource =
            (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                       JLabel.class.getName());
        textResource =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonResource =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        panelAPI = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                              JPanel.class.getName());
        layoutAPI =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelContent = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                  JPanel.class.getName());
        labelOutput = (JLabel) Beans.instantiate(getClass().getClassLoader(),
                                                 JLabel.class.getName());
        textOutput =
            (JTextField) Beans.instantiate(getClass().getClassLoader(),
                                           JTextField.class.getName());
        buttonOutput =
            (JButton) Beans.instantiate(getClass().getClassLoader(),
                                        JButton.class.getName());
        layoutContent =
            (GridBagLayout) Beans.instantiate(getClass().getClassLoader(),
                                              GridBagLayout.class.getName());
        panelFiller = (JPanel) Beans.instantiate(getClass().getClassLoader(),
                                                 JPanel.class.getName());
        layoutFiller =
            (BorderLayout) Beans.instantiate(getClass().getClassLoader(),
                                             BorderLayout.class.getName());
        radioResourceMap.setText("Presentation Object");
        radioResourceMap.setToolTipText("Use package map with source path");
        radioResourceMap.setActionCommand("");
        radioResourcePath.setText("Servlet");
        radioResourcePath.setToolTipText("Set resource document root path");
        labelResource.setText("Resource:");
        textResource.setMaximumSize(new Dimension(110, 21));
        textResource.setMinimumSize(new Dimension(110, 21));
        textResource.setPreferredSize(new Dimension(110, 21));
        textResource.setText("RESOURCE");
        buttonResource.setText("...");
        labelOutput.setText("Output:");
        textOutput.setText("OUTPUT");
        buttonOutput.setText("...");
        // API
        panelAPI.setBorder(BorderFactory.createTitledBorder("Presentaion API"));
        panelAPI.setLayout(layoutAPI);
        panelFiller.setLayout(layoutFiller);
        panelAPI.add(radioResourcePath,
                     new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        panelAPI.add(radioResourceMap,
                     new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                            GridBagConstraints.CENTER,
                                            GridBagConstraints.HORIZONTAL,
                                            new Insets(5, 5, 5, 5), 0, 0));
        // Content
        panelContent.setLayout(layoutContent);
        panelContent.setBorder(BorderFactory.createTitledBorder("Content Paths"));
        panelContent.add(labelResource,
                         new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelContent.add(textResource,
                         new GridBagConstraints(1, 0, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(5, 5, 5, 5), 100,
                                                0));
        panelContent.add(buttonResource,
                         new GridBagConstraints(2, 0, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelContent.add(labelOutput,
                         new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        panelContent.add(textOutput,
                         new GridBagConstraints(1, 1, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.WEST,
                                                GridBagConstraints.HORIZONTAL,
                                                new Insets(5, 5, 5, 5), 100,
                                                0));
        panelContent.add(buttonOutput,
                         new GridBagConstraints(2, 1, 1, 1, 0.1, 0.1,
                                                GridBagConstraints.CENTER,
                                                GridBagConstraints.NONE,
                                                new Insets(5, 5, 5, 5), 0,
                                                0));
        // Main
        this.setLayout(layoutMain);
        this.add(panelAPI,
                 new GridBagConstraints(0, 0, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 5, 5));
        this.add(panelContent,
                 new GridBagConstraints(0, 1, 1, 1, 0.1, 0.1,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.HORIZONTAL,
                                        new Insets(5, 5, 5, 5), 5, 5));
        this.add(panelFiller,
                 new GridBagConstraints(0, 2, 1, 1, 0.2, 0.2,
                                        GridBagConstraints.CENTER,
                                        GridBagConstraints.BOTH,
                                        new Insets(5, 5, 5, 5), 10, 10));
    }

    class LocalButtonListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (source == buttonResource) {
                browseForPath(textResource, "Select resource root directory");
            } else if (source == buttonOutput) {
                browseForPath(textOutput, "Select output content directory");
            }
        }

    }
    //
    class LocalRadioListener implements ActionListener {
        public void actionPerformed(ActionEvent event) {
            Object source = event.getSource();

            if (radioResourceMap.isSelected()) {
                textResource.setText("");
                textResource.setEnabled(false);
                buttonResource.setEnabled(false);
                textOutput.setText("");
                textOutput.setEnabled(false);
                buttonOutput.setEnabled(false);
            } else {
                if (project != null) {
                  textResource.setText(project.getSourcePathArray()[0]);
                }
                textResource.setEnabled(true);
                buttonResource.setEnabled(true);
                textOutput.setEnabled(true);
                buttonOutput.setEnabled(true);

            }
        }

    }
}
