/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.common.wizard.conf;

import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.swing.ExtensionFilter;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterProject;

import java.awt.Dimension;
import java.awt.GridBagLayout;
import java.awt.GridBagConstraints;
import java.awt.Insets;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.beans.Beans;
import java.io.File;

import javax.swing.BorderFactory;
import javax.swing.JFileChooser;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;


public class StartupPropertyPanel extends JPanel {
  private OtterNode node;
  private GridBagLayout layoutMain;
  private JCheckBox checkConfigure;
  private JCheckBox checkRunClass;
  private JCheckBox checkCreateJava;
  private JPanel panelConfigFile;
  private JTextField textConfigFile;
  private GridBagLayout layoutConfigFile;
  private JButton buttonSet;
  private TitledBorder borderConf;
  private LocalButtonListener buttonListener = null;

  public StartupPropertyPanel() {
    try {
      jbInit();
      pmInit();
    }
    catch(Exception e) {
      e.printStackTrace();
    }
  }

  public OtterNode getNode() {
    return node;
  }

  public void setNode(OtterNode n) {
    node = n;
  }

  private void pmInit() {
	buttonListener = new LocalButtonListener();
	buttonSet.addActionListener(buttonListener);
     if (KelpVersion.getEnhydraVersion().startsWith("4")) {
       borderConf = BorderFactory.createTitledBorder("Configuration Directory");
     } else {
       borderConf = BorderFactory.createTitledBorder("MultiServer Configuration File");
     }
     panelConfigFile.setBorder(borderConf);
  }


  private void jbInit() throws Exception {
    checkRunClass = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    checkCreateJava = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    panelConfigFile = (JPanel) Beans.instantiate(getClass().getClassLoader(), JPanel.class.getName());
    textConfigFile = (JTextField) Beans.instantiate(getClass().getClassLoader(), JTextField.class.getName());
    layoutConfigFile = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());
    buttonSet = (JButton) Beans.instantiate(getClass().getClassLoader(), JButton.class.getName());
    checkConfigure = (JCheckBox) Beans.instantiate(getClass().getClassLoader(), JCheckBox.class.getName());
    layoutMain = (GridBagLayout) Beans.instantiate(getClass().getClassLoader(), GridBagLayout.class.getName());


    checkConfigure.setText("Configure MultiServer startup");
    checkConfigure.addActionListener(new java.awt.event.ActionListener() {

      public void actionPerformed(ActionEvent e) {
        enableUI(checkConfigure.isSelected());
      }
    });

    checkRunClass.setText("Set project run parameters");
    checkCreateJava.setText("Create StartMultiserver.java");
    textConfigFile.setText("textConfFile");
    textConfigFile.setMinimumSize(new Dimension(100, 21));
    textConfigFile.setPreferredSize(new Dimension(100, 21));
    textConfigFile.setEditable(true);

    panelConfigFile.setLayout(layoutConfigFile);
    buttonSet.setActionCommand("buttonSet");
    buttonSet.setText("Set...");
    panelConfigFile.add(textConfigFile, new GridBagConstraints(0, 0, 1, 1, 1.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 150, 0));
    panelConfigFile.add(buttonSet, new GridBagConstraints(1, 0, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.NONE, new Insets(5, 5, 5, 5), 0, 0));
    this.setLayout(layoutMain);
    this.add(checkConfigure, new GridBagConstraints(0, 0, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 5, 5, 5), 0, 0));
    this.add(checkRunClass, new GridBagConstraints(0, 1, 3, 1, 0.0, 0.0
            ,GridBagConstraints.SOUTHWEST, GridBagConstraints.HORIZONTAL, new Insets(5, 30, 5, 5), 0, 0));
    this.add(checkCreateJava, new GridBagConstraints(0, 2, 3, 1, 0.0, 0.0
            ,GridBagConstraints.WEST, GridBagConstraints.HORIZONTAL, new Insets(5, 30, 5, 5), 0, 0));
    this.add(panelConfigFile, new GridBagConstraints(0, 3, 1, 1, 0.0, 0.0
            ,GridBagConstraints.CENTER, GridBagConstraints.BOTH, new Insets(5, 30, 5, 5), 0, 0));
  }

  private void enableUI(boolean enable) {
    buttonSet.setEnabled(enable);
    checkCreateJava.setEnabled(enable);
    checkRunClass.setEnabled(enable);
    textConfigFile.setEnabled(enable);
  }

  public void readProperties() {
    OtterProject project = node.getProject();

    checkConfigure.setSelected(project.isConfStartupSelected());
    checkCreateJava.setSelected(project.isConfStartupJava());
    checkRunClass.setSelected(project.isConfStartupRun());
    textConfigFile.setText(project.getConfStartupConf());
    enableUI(checkConfigure.isSelected());
  }

  public void writeProperties() {
    OtterProject project = node.getProject();
    project.setConfStartupSelected(checkConfigure.isSelected());
    project.setConfStartupJava(checkCreateJava.isSelected());
    project.setConfStartupRun(checkRunClass.isSelected());
    project.setConfStartupConf(textConfigFile.getText());
  }

  private void browseForConfig() {
    File defaultSelection = null;
    File choice = null;
    if (textConfigFile.getText().trim().length() > 0) {
      defaultSelection = new File(textConfigFile.getText());
    } else {
      defaultSelection = new File(node.getFilePath());
      defaultSelection = defaultSelection.getParentFile();
    }
    if (KelpVersion.getEnhydraVersion().startsWith("4")) {
      if (! defaultSelection.isDirectory()) {
        defaultSelection = defaultSelection.getParentFile();
      }
      choice = SwingUtil.getDirectoryChoice(this, defaultSelection,
               "Select a MultiServer Configuration directory");
    } else {
      ExtensionFilter filter;
      filter = new ExtensionFilter();
      filter.addExtension("conf");
      filter.setDescriptionTitle("MultiServer Configuration");

      choice = SwingUtil.getFileChoice(
           this, defaultSelection,
           filter, "Select a conf file");
    }
    if (choice != null) {
      textConfigFile.setText(choice.getAbsolutePath());
    }
    buttonSet.requestFocus();
  }

    class LocalButtonListener implements ActionListener {
	public void actionPerformed(ActionEvent event) {
	    Object source = event.getSource();

	    if (source == buttonSet) {
		browseForConfig();
	    }
	}

    }


}
