package org.enhydra.kelp.common.wizard.conf;

import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.FileUtil;
import org.enhydra.kelp.common.PropUtil;

import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.PrintWriter;


public class TemplateFile {
    private File       inFile = null;
    private File       outFile = null;
    private String[][] confTable = new String[0][0];

    public TemplateFile() {
    }

    /**
     * Constructor declaration
     *
     *
     * @param confNode
     *
     * @see
     */
    protected TemplateFile(OtterTemplateNode templateNode) {
        OtterProject project = templateNode.getProject();
        String[][]   rawTable = project.getConfTable();

        inFile = new File(templateNode.getFilePath());
        outFile = templateNode.getOutputFile();
        confTable = new String[rawTable.length][2];
        for (int i = 0; i < rawTable.length; i++) {
            confTable[i][0] = rawTable[i][0];
            confTable[i][1] = PropUtil.expandPathRelativeToProject(project,
                    rawTable[i][1]);
        }

        // make any needed directories.
        outFile.getParentFile().mkdirs();
    }

    /**
     * Method declaration
     *
     *
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     *
     * @see
     */
    protected void backupOutFile()
            throws java.io.FileNotFoundException, java.io.IOException {
        if (outFile.exists()) {
            FileUtil.copyFile(outFile,
                              new File(outFile.getAbsolutePath() + ".bak"));
        }
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void deleteOutFile() {
        if (outFile.exists()) {
            outFile.delete();
        }
    }

    /**
     * Method declaration
     *
     *
     * @exception java.io.FileNotFoundException
     * @exception java.io.IOException
     *
     * @see
     */
    protected void copyInFileToOutFile()
            throws java.io.FileNotFoundException, java.io.IOException {
        FileUtil.copyFile(inFile, outFile);
    }

    /**
     * Method declaration
     *
     *
     * @see
     */
    protected void processOutFile() {
        String[] find = new String[confTable.length];
        String[] replace = new String[confTable.length];
        boolean[] longFile = new boolean[confTable.length];

        for (int i = 0; i < confTable.length; i++) {
            find[i] = confTable[i][0];
            replace[i] = confTable[i][1];
            longFile[i] = false;
            // if it has embedded spaces check if it may be a file.
            if (replace[i].trim().indexOf(" ") > 0) {
              File file = new File(replace[i]);
              if (file.getParentFile() != null) {
                 if (file.getParentFile().getParentFile() != null) {
                    if (file.getParentFile().getParentFile().exists()) {
                      replace[i] = "\"" + replace[i] + "\"";
                      replace[i] = replace[i].replace('\\','/');
                      longFile[i] = true;
                    }
                 }
              }
            }
        }
        searchAndReplace(outFile.getAbsolutePath(), find, replace, longFile);
    }

    /**
     * Method declaration
     *
     *
     * @param filename
     * @param find
     * @param replace
     *
     * @see
     */
    public void searchAndReplace(String filename, String[] find,
                                  String[] replace,
                                  boolean[] longFile) {
        BufferedReader in = null;
        PrintWriter    out = null;
        File           file = new File(filename);
        File           temp = null;

        try {
            temp = File.createTempFile("kelp", ".tmp",
                                       new File(file.getParent()));
            String tempName = temp.getAbsolutePath();

            temp.delete();
            temp = null;
            file.renameTo(new File(tempName));
            file = null;
            temp = new File(tempName);
            in = new BufferedReader(new FileReader(temp));
            out = new PrintWriter(new FileWriter(new File(filename)));
            String line = in.readLine();

            while (line != null) {
                line = lineSearchAndReplace(line,
                                            createReplacementSet(find,
                                            replace, longFile));
                out.println(line);
                line = in.readLine();
            }
            in.close();
            out.close();
        } catch (java.io.IOException e) {
            e.printStackTrace();
        }
        temp.delete();
        in = null;
        out = null;
        file = null;
        temp = null;
    }

    /**
     * Method declaration
     *
     *
     * @param line
     *
     * @return
     *
     * @see
     */

    // See org.enhydra.codegen.TemplateDrivenGenerator::lineSearchAndReplace()
    private String lineSearchAndReplace(String in,
                                        LocalReplacement[] replacements) {
        String text = in;
        String prefix = new String();
        String suffix = new String();

        for (int i = 0; i < replacements.length; i++) {
            int indexOf = text.indexOf(replacements[i].getFind());
            int lastIndex = -1;
            int count = 0;

            while (indexOf != -1 && indexOf > lastIndex && count < 1000) {

                // Catch recursive replacements by maxing out at
                // after the 1000 replacement
                count++;
                prefix = text.substring(0, indexOf);
                suffix = text.substring(indexOf
                                        + replacements[i].getFind().length());
                StringBuffer buf = new StringBuffer();

                buf.append(prefix);
                for (int j = 0; j < replacements[i].getReplaceWith().length;
                        j++) {
                    String rep = replacements[i].getReplaceWith()[j];
                    if (replacements[i].isLongFileName()) {
                      if (suffix.startsWith("/")
                        && rep.endsWith("\"")) {
                        suffix = suffix + "\"";
                        rep = rep.substring(0, (rep.length()-1) );
                      }
                    }
                    buf.append(rep);
                }
                buf.append(suffix);
                text = buf.toString();
                indexOf = text.indexOf(replacements[i].getFind());
            }
        }
        return text;
    }

    /**
     * Method declaration
     *
     *
     * @param prefix
     * @param replacement
     * @param suffix
     *
     * @return
     *
     * @see
     */
    private String transformFileValues(String prefix, String replacement,
                                       String suffix) {

        // Quote and use unix style separators
        // to support long file names on Windows.
        String line = prefix + replacement + suffix;
        String value = replacement + suffix;
        File   file;

        if (prefix.trim().endsWith("=")) {
            file = new File(replacement);
            file = new File(file.getParent());
            file = new File(file.getParent());
            if (file.exists() && file.isDirectory()) {
                value = value.replace('\\', '/');
                value = "\"" + value + "\"";
            }
            line = prefix + value;
        }
        return line;
    }

    /**
     * Method declaration
     *
     *
     * @param find
     * @param replaceWith
     *
     * @return
     *
     * @see
     */
    private LocalReplacement[] createReplacementSet(String[] find,
            String[] replaceWith, boolean[] longFile) {
        LocalReplacement[] replacements = new LocalReplacement[find.length];

        for (int i = 0; i < find.length; i++) {
            replacements[i] = new LocalReplacement(find[i], replaceWith[i], longFile[i]);
        }
        return replacements;
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision: 1.1 $
     */
    class LocalReplacement {
        private String   find = new String();
        private String[] replaceWith = new String[1];
        private boolean longFile = false;

        /**
         * Constructor declaration
         *
         *
         * @param find
         * @param replaceWith
         *
         * @see
         */
        private LocalReplacement(String f, String r, boolean b) {
            find = f;
            replaceWith[0] = r;
            longFile = b;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        private String getFind() {
            return find;
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        private String[] getReplaceWith() {
            return replaceWith;
        }

        private boolean isLongFileName() {
          return longFile;
        }

    }
}
