/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jbuilder.properties;

// Kelp imports
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.jbuilder.node.PrimeHTMLFileNode;
import org.enhydra.kelp.common.node.OtterXMLCNode;

// JBuilder Foundation imports
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.primetime.ide.Browser;
import com.borland.primetime.node.Node;
import com.borland.primetime.node.LightweightNode;
import com.borland.primetime.node.TextFileNode;
import com.borland.primetime.node.Project;
import com.borland.primetime.properties.PropertyPageFactory;
import com.borland.primetime.properties.PropertyPage;
import com.borland.primetime.properties.PropertyManager;
import com.borland.primetime.properties.PropertyGroup;

public class XMLCProperties implements PropertyGroup {

  public static final String GENERATE_CLASS = "Generate";

  public XMLCProperties() {
  }


  /**
   * Initialize this OpenTool
   */
  public static void initOpenTool(byte majorVersion, byte minorVersion) {
	PropertyManager.registerPropertyGroup(new XMLCProperties());
  }

  /**
   * Returns a PropertyPageFactory for the project or for an
   * HTMLFileNode. Null if topic is something else.
   *
   * TODO: ?? Add XMLFileNode ??
   */
  public PropertyPageFactory getPageFactory(final Object topic) {
    PropertyPageFactory factory = null;
    Project project = null;

    if (topic instanceof TextFileNode) {
      TextFileNode textNode = (TextFileNode) topic;
      project = textNode.getProject();
      if (project == null) {
      	 // Skip
      } else if (textNode.getUrl().getFileExtension().equalsIgnoreCase("wml") ||
                 textNode instanceof HTMLFileNode) {
        // Node property page to set your XMLC properties
        factory = (new PropertyPageFactory("Enhydra XMLC") {
          public PropertyPage createPropertyPage() {
            return new XMLCNodePropertyPage(topic);
          }
        });
      }
    } else if (topic instanceof LightweightNode) {
        // Package or Folder node
        LightweightNode lightNode = null;
        lightNode = (LightweightNode) topic;
        project = lightNode.getProject();
        if (project == null) {
       	   // Skip
        } else {
          factory = (new PropertyPageFactory("Enhydra XMLC") {
            public PropertyPage createPropertyPage() {
              return new XMLCPackagePropertyPage(topic);
            }
          });
        }
    } else if (topic instanceof Project) {
        // Project property page to set your XMLC properties
        factory = (new PropertyPageFactory("Enhydra XMLC") {
          public PropertyPage createPropertyPage() {
            return new XMLCProjectPropertyPage(topic);
          }
        });
    }
    return factory;
  }

  public void initializeProperties() {
  }
}
