/*
 * Enhydra Java Application Server Project
 *
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 *
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See
 * the License for the specific terms governing rights and limitations
 * under the License.
 *
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 *
 * Contributor(s):
 *
 * $Id$
 */
package org.enhydra.kelp.jbuilder.wizard.importer;

// Kelp imports
import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.swing.SwingUtil;
import org.enhydra.kelp.common.swing.IconSet;
import org.enhydra.kelp.common.ValidationException;
import org.enhydra.kelp.common.wizard.CodeImporter;
import org.enhydra.kelp.common.wizard.importer.ImportMapPanel;
import org.enhydra.kelp.common.wizard.importer.RootPanel;
import org.enhydra.kelp.common.wizard.importer.PathPanel;
import org.enhydra.kelp.common.wizard.importer.ImporterPanel;
import org.enhydra.kelp.jbuilder.wizard.AbstractBasicWizard;
import org.enhydra.kelp.jbuilder.node.PrimeProject;
import org.enhydra.kelp.jbuilder.node.PrimeProject;

// JBuilder Foundation imports
import com.borland.jbuilder.node.HTMLFileNode;
import com.borland.jbuilder.node.JBProject;
import com.borland.primetime.node.Node;
import com.borland.primetime.wizard.BasicWizard;
import com.borland.primetime.wizard.BasicWizardPage;
import com.borland.primetime.wizard.Wizard;
import com.borland.primetime.wizard.WizardAction;
import com.borland.primetime.wizard.WizardHost;
import com.borland.primetime.wizard.WizardManager;
import com.borland.primetime.wizard.WizardPage;
import com.borland.primetime.util.VetoException;

// Standard imports
import java.awt.BorderLayout;
import java.io.File;
import javax.swing.JPanel;
import javax.swing.JOptionPane;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%p
 */
public class ImportWizard extends AbstractBasicWizard {
    private static String WIZARD_NAME = "Enhydra Import";

    private CodeImporter       importer = null;
    private OtterProject       project = null;
    private final String       HELP_TOPIC =
        "file:../kelp1.2/doc/kelp.html#import";
    public static WizardAction action = new LocalWizardAction();

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision$
     */
    static class LocalWizardAction extends WizardAction {

        /**
         * Constructor declaration
         *
         *
         * @see
         */
        public LocalWizardAction() {
            super(ImportWizard.WIZARD_NAME + "...", 'p',
                  "Import source files into current project",
                  IconSet.getSmallIcon(), IconSet.getLargeIcon(), false);
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        protected Wizard createWizard() {
            ImportWizard wizard = new ImportWizard();

            return wizard;
        }

        /**
         * Method declaration
         *
         *
         * @param source
         *
         * @see
         */
        public void update(Object source) {
            setEnabled(AbstractBasicWizard.enableMenu);
        }

    }
    ;

    /**
     * Method declaration
     *
     *
     * @exception VetoException
     *
     * @see
     */
    protected void finish() throws VetoException {
        importer.importFilesFromRoot(project);
    }

    /**
     * Method declaration
     *
     *
     * @param majorVersion
     * @param minorVersion
     *
     * @see
     */
    public static void initOpenTool(byte majorVersion, byte minorVersion) {
        WizardManager.registerWizardAction(action);
    }

    /**
     * Method declaration
     *
     *
     * @param currentPage
     * @param host
     *
     * @see
     */
    public String getHelpTopic() {
        return HELP_TOPIC;
    }

    /**
     * Method declaration
     *
     *
     * @return
     *
     * @see
     */
    public String getDisplayName() {
        return ImportWizard.WIZARD_NAME;
    }

    public static String getName() {
        return ImportWizard.WIZARD_NAME;
    }



    /**
     * Method declaration
     *
     *
     * @param host
     *
     * @return
     *
     * @see
     */
    public WizardPage invokeWizard(WizardHost host) {
         WizardPage page         = null;
        JBProject  nativeProject = null;
        boolean    invokeIt = true;
        nativeProject = (JBProject) host.getBrowser().getActiveProject();
        if (!isProjectEmpty(nativeProject)) {
            int choice =
                JOptionPane.showConfirmDialog(host.getDialogParent(),
                                              "Project already contains entries. Importing \n"
                                              + "files into an non-empty project may cause \n"
                                              + "unexpected results. \n\n"
                                              + "Do you want to continue?",
                                              getDisplayName()
                                              + " Wizard Error", JOptionPane.WARNING_MESSAGE);

            if (choice == JOptionPane.CANCEL_OPTION) {
                invokeIt = false;
            }
        }
        if (invokeIt) {
            importer = new CodeImporter();
            project = new PrimeProject(nativeProject);
            String projectPath = project.getFilePath();
            if (projectPath != null && projectPath.trim().length() > 0) {
              File projectFile = new File(projectPath);
              if (projectFile.getParentFile() != null) {
                try {
                  importer.setRoot(projectFile.getParentFile());
                } catch (ValidationException e) {
                  e.printStackTrace();
                }
              }
            }
            LocalWizardPage rootPage = new LocalWizardPage();
            LocalWizardPage pathPage = new LocalWizardPage();
            LocalWizardPage mapPage = new LocalWizardPage();

            rootPage.setPanel(new RootPanel());
            rootPage.setImporter(importer);
            pathPage.setPanel(new PathPanel());
            pathPage.setImporter(importer);
            mapPage.setPanel(new ImportMapPanel());
            mapPage.setImporter(importer);
            addWizardPage(rootPage);
            addWizardPage(pathPage);
            addWizardPage(mapPage);
            page = super.invokeWizard(host);
        }
        return page;
    }

    /**
     * Method declaration
     *
     *
     * @param nativeProject
     *
     * @return
     *
     * @see
     */
    private boolean isProjectEmpty(JBProject nativeProject) {
        Node[]  nodes = null;
        boolean empty = true;

        if (nativeProject != null) {
            nodes = nativeProject.getChildren();
        }
        if (nodes != null) {
            if (nodes.length > 1) {
                empty = false;
            } else if (nodes.length == 1) {
                if (!(nodes[0] instanceof HTMLFileNode)) {
                    empty = false;
                }
            }
        }
        return empty;
    }

    /**
     * Class declaration
     *
     *
     * @author Paul Mahar
     * @version $Revision$
     */
    class LocalWizardPage extends BasicWizardPage {
        private ImporterPanel panel;

        /**
         * Constructor declaration
         *
         *
         * @see
         */
        public LocalWizardPage() {
            BorderLayout layout = new BorderLayout();

            layout.setHgap(5);
            layout.setVgap(5);
            this.setLayout(layout);
            this.setLargeIcon(IconSet.getOtterWizard());
        }

        /**
         * Method declaration
         *
         *
         * @param host
         *
         * @see
         */
        public void activated(WizardHost host) {
            panel.activated();
        }

        /**
         * Method declaration
         *
         *
         * @exception VetoException
         *
         * @see
         */
        public void checkPage() throws VetoException {
            try {
                panel.checkPage();
            } catch (ValidationException e) {
                JOptionPane.showMessageDialog(getWizardHost().getDialogParent(),
                                              e.getMessage(),
                                              getDisplayName(),
                                              JOptionPane.ERROR_MESSAGE);
                throw new VetoException(e.toString());
            }
        }

        /**
         * Method declaration
         *
         *
         * @param importer
         *
         * @see
         */
        public void setImporter(CodeImporter importer) {
            panel.setImporter(importer);
        }

        /**
         * Method declaration
         *
         *
         * @return
         *
         * @see
         */
        public CodeImporter getImporter() {
            return panel.getImporter();
        }

        /**
         * Constructor declaration
         *
         *
         * @param panel
         *
         * @see
         */
        protected void setPanel(ImporterPanel panel) {
            this.panel = panel;
            this.setPageTitle(panel.getPageTitle());
            this.setInstructions(panel.getInstructions());
            this.add(panel, BorderLayout.CENTER);
        }

        /**
         * Constructor declaration
         *
         *
         * @see
         */
        protected JPanel getPanel() {
            return panel;
        }

    }
}
