/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jdeveloper.node;

import org.enhydra.kelp.common.node.OtterNodeFactory;
import org.enhydra.kelp.common.node.OtterProject;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterFolderNode;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterHTMLFileNode;
import org.enhydra.kelp.common.node.OtterImageFileNode;
import org.enhydra.kelp.common.node.OtterJavaFileNode;
import org.enhydra.kelp.common.node.OtterTextFileNode;
import org.enhydra.kelp.common.node.OtterWMLFileNode;
/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OracleNodeFactory extends OtterNodeFactory {

    /**
     * Method declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param name
     *
     * @return
     *
     * @see
     */
    public OtterFolderNode createFolderNode(OtterProject otterProject,
					    OtterNode otterNode,
					    String name) {
	OtterFolderNode folder = new OracleFolderNode(otterProject, otterNode,
						   name);

	return folder;
    }

    /**
     * Method declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @return
     *
     * @see
     */
    public OtterJavaFileNode createJavaFileNode(OtterProject otterProject,
						OtterNode otterNode,
						String file) {
	OtterJavaFileNode javaFile = new OracleJavaFileNode(otterProject,
		otterNode, file);

	return javaFile;
    }

    /**
     * Method declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @return
     *
     * @see
     */
    public OtterHTMLFileNode createHTMLFileNode(OtterProject otterProject,
						OtterNode otterNode,
						String file) {
	OtterHTMLFileNode html = new OracleHTMLFileNode(otterProject, otterNode,
		file);

	return html;
    }

    /**
     * Method declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @return
     *
     * @see
     */
    public OtterImageFileNode createImageFileNode(OtterProject otterProject,
	    OtterNode otterNode, String file) {
	OtterImageFileNode image = new OracleImageFileNode(otterProject,
		otterNode, file);

	return image;
    }

    /**
     * Method declaration
     *
     *
     * @param otterProject
     * @param otterNode
     * @param file
     *
     * @return
     *
     * @see
     */
    public OtterTextFileNode createTextFileNode(OtterProject otterProject,
						OtterNode otterNode,
						String file) {
	OtterTextFileNode text = new OracleTextFileNode(otterProject, otterNode,
		file);

	return text;
    }

    public OtterWMLFileNode createWMLFileNode(OtterProject otterProject,
				              OtterNode otterNode,
				              String file) {
	OtterWMLFileNode text = new OracleWMLFileNode(otterProject, otterNode,
		file);

	return text;
    }


    public OtterTemplateNode createTemplateNode(OtterProject otterProject,
						OtterNode otterNode,
						String file) {
	OtterTemplateNode template = null;
        template = new OracleTemplateNode(otterProject, otterNode,
		file);

	return template;
    }


    public OtterJavaFileNode[] replaceGeneratedSource(
           OtterProject project,
           OtterNode node, String[] files) {
       // nothing extra for Win32.
       OtterJavaFileNode[] nodes = new OtterJavaFileNode[files.length];
       for (int i = 0; i < files.length ; i++ ) {
         nodes[i] = createJavaFileNode(project, node, files[i]);
       }
       return nodes;
    }


}

