/*
 * Enhydra Java Application Server Project
 * 
 * The contents of this file are subject to the Enhydra Public License
 * Version 1.1 (the "License"); you may not use this file except in
 * compliance with the License. You may obtain a copy of the License on
 * the Enhydra web site ( http://www.enhydra.org/ ).
 * 
 * Software distributed under the License is distributed on an "AS IS"
 * basis, WITHOUT WARRANTY OF ANY KIND, either express or implied. See 
 * the License for the specific terms governing rights and limitations
 * under the License.
 * 
 * The Initial Developer of the Enhydra Application Server is Lutris
 * Technologies, Inc. The Enhydra Application Server and portions created
 * by Lutris Technologies, Inc. are Copyright Lutris Technologies, Inc.
 * All Rights Reserved.
 * 
 * Contributor(s):
 * 
 * $Id$
 */

package org.enhydra.kelp.jdeveloper.node;

import org.enhydra.kelp.KelpVersion;
import org.enhydra.kelp.common.PropUtil;
import org.enhydra.kelp.common.node.OtterTemplateNode;
import org.enhydra.kelp.common.node.OtterNode;
import org.enhydra.kelp.common.node.OtterSelectionNode;
import org.enhydra.kelp.common.node.OtterProject;
import borland.jbuilder.addin.Project;
import borland.jbuilder.addin.Node;
import borland.jbuilder.addin.Url;
import java.io.File;

/**
 * Class declaration
 *
 *
 * @author
 * @version %I%, %G%
 */
public class OracleTemplateNode extends OracleNode implements OtterTemplateNode {
    private Exception exception = null;

    public OracleTemplateNode(Node nativeNode) {
      super(nativeNode);
    }

    /**
     * Constructor declaration
     *
     *
     * @param otterProject
     * @param node
     * @param file
     *
     * @see
     */
    public OracleTemplateNode(OtterProject otterProject, OtterNode node,
			   String file) {
	Node    nativeNode = null;
	Project nativeProject = null;
	Url     url = null;

	if (otterProject.getNativeProject() instanceof Project) {
	    nativeProject = (Project) otterProject.getNativeProject();
	}

	url = new Url(file);
	nativeNode = nativeProject.createNode(url, true);

	nativeProject.addNode(nativeNode);
	setNativeNode(nativeNode);
    }

    public boolean isServer() {
      String in = getProperty(OtterTemplateNode.SERVER);
      return PropUtil.stringToBoolean(in, false);
    }

    public void setServer(boolean b) {
      setProperty(OtterTemplateNode.SERVER, PropUtil.booleanToString(b));
    }


  public boolean isSelected() {
    String in = getProperty(OtterSelectionNode.SELECTED);
    return PropUtil.stringToBoolean(in, false);
  }

  public void setSelected(boolean b) {
   setProperty(OtterSelectionNode.SELECTED, PropUtil.booleanToString(b));
  }


    /**
     * Save Exception information for reporting. This lets exceptions from
     * multiple nodes to be reported in a batch.
     *
     * @param e
     * An exception that occured while invoking XMLC.
     */
    public void setException(Exception e) {
	exception = e;
    }

    /**
     * Get an exception that occured when invoking XMLC. Null if the node
     * has not been compiled or was compiled without error.
     */
    public Exception getException() {
	return exception;
    }


  public File getOutputFile() {
    File out = null;
    String absolute = null;

    if (KelpVersion.getEnhydraVersion().startsWith("4") && isServer()) {
      absolute = getProject().getServletConfPath() +
          File.separator + "servlet.conf";
    } else {
      out = new File(getFilePath());
      absolute = getProject().getServletConfPath() +
          File.separator + out.getName();
      absolute = absolute.substring(0,absolute.length()-3);
      out = new File(absolute);
    }
    out = new File(absolute);
    return out;
  }


}


